package cn.iocoder.yudao.module.system.dal.mysql.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.datapermission.core.annotation.DataPermission;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.QueryWrapperX;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogReadUserRespDTO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 日志实例的拓展 Mapper
 *
 * @author 符溶馨
 */
@Mapper
public interface LogInstanceMapper extends BaseMapperX<LogInstanceDO> {


    default PageResult<LogInstanceDO> getUpLog(Long userId, Long formId) {

        return selectPage(new PageParam().setPageNo(1).setPageSize(1), new LambdaQueryWrapperX<LogInstanceDO>()
                .eq(LogInstanceDO::getStartUserId, userId)
                .eqIfPresent(LogInstanceDO::getFormId, formId)
                .orderByDesc(LogInstanceDO::getCreateTime));
    }

    default Long getLogByFormId(Long formId, Long userId, List<String> dateList) {

        return selectCount(new LambdaQueryWrapperX<LogInstanceDO>()
                .eq(LogInstanceDO::getFormId, formId)
                .eq(LogInstanceDO::getStartUserId, userId)
                .in(LogInstanceDO::getTime, dateList));
    }

    List<LogInstanceRespVO> selectPageResult( @Param("reqVO") LogInstancePageReqVO reqVO,
                                              @Param("userId") Long userId,
                                              @Param("ids") List<Long> ids);

    default PageResult<LogInstanceDO> selectPage(LogInstancePageReqVO reqVO, Long userId,
                                                  Integer pagingType, List<Long> userIds) {

        LambdaQueryWrapperX<LogInstanceDO> queryWrapper = new LambdaQueryWrapperX<LogInstanceDO>()
                .eqIfPresent(LogInstanceDO::getFormId, reqVO.getFormId())
                .eqIfPresent(LogInstanceDO::getDeptId, reqVO.getDeptId())
                .eqIfPresent(LogInstanceDO::getStartUserId, reqVO.getStartUserId())
                .betweenIfPresent(LogInstanceDO::getTime, reqVO.getCreateTime());

        if (pagingType == 0) {
            queryWrapper.ne(LogInstanceDO::getStartUserId, userId);
        } else if (pagingType == 1) {
            queryWrapper.eq(LogInstanceDO::getStartUserId, userId);
        }

        if (reqVO.getIsBoss() != null && reqVO.getIsProduce() == null) {
            queryWrapper.inIfPresent(LogInstanceDO::getStartUserId, userIds);
        }

        return selectPage(reqVO, queryWrapper);

    }

    @DataPermission(enable = false)
    List<LogReadUserRespDTO> selectRaedUser(@Param("userId")Long userId, @Param("deptId")Long deptId);

    /**
     * 获取上一篇下一篇
     *
     * @param dto
     * @param id
     * @param userId
     * @param type
     * @return
     */
    LogInstanceRespVO getNextOrUp(@Param("reqVO") LogInstancePageReqVO dto,
                                  @Param("id") Long id, @Param("userId") Long userId,
                                  @Param("type") Integer type,
                                  @Param("pagingType") Integer pagingType,
                                  @Param("userIds") List<Long> userIds);
}